﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Calculator
{
    internal class DelegateCommand : ICommand
    {
        /// <summary>
        /// Выполняемые операции.
        /// </summary>
        private Action<object> _execute;
        /// <summary>
        /// Может ли команда выполниться.
        /// </summary>
        private Func<object, bool> _canExecute;

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        /// <summary>
        /// Реализация объекта, определяющего созданные команды.
        /// </summary>
        /// <param name="execute">Выполняемые командой операции.</param>
        /// <param name="canExecute">Может ли команда выполниться.</param>
        public DelegateCommand(Action<object> execute, Func<object,bool> canExecute = null) 
        { 
            _execute = execute;
            _canExecute = canExecute;
        }

        public bool CanExecute(object parameter)
        {
            return _canExecute == null || _canExecute(parameter);
        }

        public void Execute(object parameter)
        {
            _execute(parameter);
        }
    }
}
